<template>
  <Popup
    ref="pop"
    :title="title"
    :shadow="!isEdit"
    :left="left"
    width="390px"
    showBtn="true"
    @yes="addData"
    @cancel="close(false)"
  >
    <el-form ref="form" :model="data" :rules="rules" label-width="100px">
      <el-form-item label="名称:">
        <el-input v-model="data.name"></el-input>
      </el-form-item>
      <el-form-item label="地址:" prop="url">
        <el-input
          v-model="data.url"
          :title="data.url"
          @change="edit('url')"
        ></el-input>
      </el-form-item>
      <el-form-item label="类型:">
        <el-select v-model="data.type" @change="edit('type')">
          <el-option label="mp4" value="mp4"></el-option>
          <el-option label="hls" value="hls"></el-option>
          <el-option label="flv" value="flv"></el-option>
        </el-select>
      </el-form-item>
      <el-form-item label="相机位置:">
        <el-button type="primary" size="mini" @click="thisCamera"
          >当前视角</el-button
        >
        <el-button type="primary" size="mini" @click="pick('cameraPosition')"
          >图上选点</el-button
        >
      </el-form-item>
      <el-form-item label="经度:">
        <el-input-number
          v-model="data.cameraPosition.lon"
          :step="0.00001"
          @change="edit('cameraPosition')"
        ></el-input-number>
      </el-form-item>
      <el-form-item label="纬度:">
        <el-input-number
          v-model="data.cameraPosition.lat"
          :step="0.00001"
          @change="edit('cameraPosition')"
        ></el-input-number>
      </el-form-item>
      <el-form-item label="高度:">
        <el-input-number
          v-model="data.cameraPosition.height"
          :step="0.1"
          @change="edit('cameraPosition')"
        ></el-input-number>
      </el-form-item>
      <el-form-item label="四周方向:">
        <el-slider
          class="heading-slider"
          v-model="data.heading"
          :max="360"
          @input="edit('heading')"
        >
        </el-slider>
        <el-button type="primary" size="mini" @click="pick('position')"
          >图上选点</el-button
        >
      </el-form-item>
      <el-form-item label="俯仰角度:">
        <el-slider
          v-model="data.pitch"
          :max="89"
          :min="-89"
          @input="edit('pitch')"
        >
        </el-slider>
      </el-form-item>
      <el-form-item label="水平张角:">
        <el-input-number
          v-model="data.horizonAngle"
          :step="0.1"
          @change="edit('horizonAngle')"
        ></el-input-number>
      </el-form-item>
      <el-form-item label="垂直张角:">
        <el-input-number
          v-model="data.verticalAngle"
          :step="0.1"
          @change="edit('verticalAngle')"
        ></el-input-number>
      </el-form-item>
      <el-form-item label="羽化比例:" v-if="!video2D">
        <el-input-number
          v-model="data.offset"
          :step="0.1"
          @change="edit('offset')"
        ></el-input-number>
      </el-form-item>
      <el-form-item label="透明度:">
        <el-slider
          v-model="data.alpha"
          :max="1"
          :step="0.01"
          @input="edit('alpha')"
        ></el-slider>
      </el-form-item>
      <el-form-item label="辅助线:">
        <el-switch v-model="data.useLine" @change="edit('useLine')"></el-switch>
      </el-form-item>
      <el-form-item label="最近投射:" v-if="!video2D">
        <el-input-number
          v-model="data.near"
          :step="0.1"
          @change="edit('near')"
        ></el-input-number>
      </el-form-item>
      <el-form-item label="最远投射:">
        <el-input-number
          v-model="data.far"
          :step="0.1"
          @change="edit('far')"
        ></el-input-number>
      </el-form-item>
    </el-form>
  </Popup>
</template>

<script>
import Popup from "@tools/Popup";
let video, handler;
export default {
  name: "Video",
  components: {
    Popup,
  },
  data() {
    return {
      title: "视频投影编辑",
      left: "calc(100% - 400px)",
      isEdit: false,
      defaultEditData: undefined,
      video2D: false,
      data: {
        name: "",
        type: "mp4",
        url: "",
        cameraPosition: {
          lon: 110,
          lat: 32,
          height: 0,
        },
        heading: 0,
        pitch: 0,
        horizonAngle: 60, //相机水平视角
        verticalAngle: 40, //相机垂直视角
        offset: 0.1, //视频四周羽化比例
        near: 1, //最近投射面距离
        far: 1000, //最远投射面距离
        alpha: 0.9, //透明度
        useLine: true, //是否试用辅助线
      },
      // 表单验证规则
      rules: {
        url: [{ required: true, message: "请输入视频地址", trigger: "blur" }],
      },
    };
  },
  mounted() {},
  methods: {
    // 关闭弹窗
    close(isCloseBtn, isSave) {
      if (!isSave && this.defaultEditData && video) {
        this.restore();
        this.defaultEditData = undefined;
      }
      video = undefined;

      handler && handler.destroy();
      handler = undefined;

      // 重置data值
      Object.assign(this.$data, this.$options.data());
      !isCloseBtn && this.$refs.pop.close();
    },
    // 打开弹窗
    open(type, editData) {
      this.close(true);

      this.type = type;
      this.isEdit = !!editData;
      if (this.isEdit) {
        // 获取图层
        video = sgworld._treeTool.getData(editData.id);
        video.item && (video = video.item);
        editData.cameraPosition.x &&
          editData.cameraPosition.y &&
          editData.cameraPosition.z &&
          (editData.cameraPosition = sgworld.Core.toDegrees(
            editData.cameraPosition
          ));
        // 合并参数
        sgworld.Core.extend(this.data, editData, true, true);
        this.data.id = editData.id;
        this.defaultEditData = editData;
        this.video2D = editData.video2D;
      }

      this.$refs.pop.open();
    },
    // 添加数据
    addData() {
      this.$refs.form.validate((valid) => {
        // 验证通过
        if (valid) {
          let data = {
            sourceType: this.type,
            ...this.data,
          };
          this.$emit("success", data, this.isEdit);
          this.close(false, true);
        }
      });
    },
    // 编辑
    edit(editType) {
      if (this.isEdit) {
        let data = [];
        data[editType] = this.data[editType];
        video.updateVideo(data);
      }
    },
    thisCamera() {
      let cameraDegrees = sgworld.Navigate.getDegrees();
      Object.assign(this.data.cameraPosition, cameraDegrees);
      let degrees = sgworld.Navigate.getMouseDegrees({
        x: Viewer.container.offsetWidth / 2,
        y: Viewer.container.offsetHeight / 2,
      });
      let data = {
        cameraPosition: Cesium.Cartesian3.fromDegrees(
          cameraDegrees.lon,
          cameraDegrees.lat,
          cameraDegrees.height
        ),
      };
      if (degrees) {
        data.position = Cesium.Cartesian3.fromDegrees(
          degrees.lon,
          degrees.lat,
          degrees.height
        );
        data.far =
          Cesium.Cartesian3.distance(data.cameraPosition, data.position) * 1.1;
        this.data.far = data.far;
      }
      video.updateVideo(data);
      this.data.heading = video.thisVideo.heading;
      this.data.pitch = video.thisVideo.pitch;
    },
    pick(type) {
      handler && handler.destroy();
      handler = new Cesium.ScreenSpaceEventHandler(Viewer.scene.canvas);
      this.$message("在场景内点击左键拾取坐标更新位置");
      handler.setInputAction((event) => {
        let degrees = sgworld.Navigate.getMouseDegrees(event);
        type === "cameraPosition" && Object.assign(this.data[type], degrees);
        video.updateVideo({
          [type]: Cesium.Cartesian3.fromDegrees(
            degrees.lon,
            degrees.lat,
            degrees.height
          ),
        });
        this.data.heading = video.thisVideo.heading;
        this.data.pitch = video.thisVideo.pitch;
        handler.destroy();
      }, Cesium.ScreenSpaceEventType.LEFT_CLICK);
    },
    restore() {
      video.updateVideo(this.defaultEditData);
    },
  },
};
</script>

<style scoped lang="less">
.el-form {
  margin-top: 20px;
  margin-right: 10px;
  width: 380px;

  /deep/ .el-form-item__label {
    color: #fff;
    font-size: 18px;
  }

  .heading-slider {
    width: 157px;
    display: inline-block;
    vertical-align: middle;
  }

  .el-select {
    width: 100%;
  }
}
</style>
